c ******************************************************************
      program eosfit
c ******************************************************************
c
c tries to fit the Eos a-H distribution...
c
c - Yarko dispersion,
c - simple Poissonian model for reorientations,
c - YORP effect
c
c written by D Vokrouhlicky, Jan 04
c querries vokrouhl@mbox.cesnet.cz
c ******************************************************************
      implicit real*8 (a-h,o-z)
c      include 'astrco.h'
c particles ......................
c npm         ... maximum number of particles
c parlis(n,1) ... R particle (in meters)
c parlis(n,2) ... a (AU)
c parlis(n,3) ... e
c parlis(n,4) ... \gamma particle
c parlis(n,5) ... \omega particle
c Some astronomical constants
      parameter (au=1.4959787d11,gms=1.327124d20,gravco=6.673d-11)
      parameter (npm=10000,namax=1000,ibim=50,ntg=100,ncbm=100)
      character*1 ifi
      dimension parlis0(npm,5),parlis(npm,5),parlisini(npm)
      dimension parlisg(npm,5)
      dimension hobs(npm),aobs(npm),dobs(npm)
      dimension biasm(ibim),biasf(ibim),ibina(namax)
      dimension eps(91),taus0(91),taue0(91),taus(91),taue(91)
      dimension fitarget(ntg),itargetm(ntg)
      dimension cbin(ncbm)
c distribution of the rotation frequencies (end) .
      character*20 inpfi(3)
      common /co/ twopi,ompr,ds3,rotmin,rotmax
c ------------------------------------------------------------------
c initializations & co.
      pi=dacos(-1.d0)
      twopi=2.d0*pi
      ds3=dsqrt(3.d0)
      degrad=pi/180.d0
      beta=1.333333d0
      rotmin=3.d0
      rotmax=12.d0
      rotmean=6.d0
c - get data of the YORP torques
      open(1,file='yorpm.dat',status='old')
      do j=1,91
       read(1,*)eps(j),taus0(j),taue0(j)
      enddo
      close(1)
c - Gaussian distribution model for the rotation rates
      rotmin=rotmin*3600.d0
      rotmax=rotmax*3600.d0
      ommax=twopi/rotmin
      ommin=twopi/rotmax
      rotp=rotmean*3600.d0
      omp=twopi/rotp
      ompr=dsqrt(1.5d0)*omp
c - YORP stuff...
      perilim=1500.d0
      pericrit=2.d0
      omlim=twopi/3600.d0/perilim
      omcrit=twopi/3600.d0/pericrit
      dom=(omcrit-omlim)/dfloat(nom-1)
c Initialize the random-number generator
      idum0=-75485
      xi=ran3(idum0)
c - get family from data file
      open(1,file='eos.dat.55',status='old')
      open(2,file='eosSD.dat',status='unknown')
c choose p_V !!!
      albedo=0.133d0
      i=1
      a0=3.02d0
      volpb=0.d0
 10   continue
       read(1,*,end=20,err=20)aobs(i),e,skl,hobs(i)
       dobs(i)=1329.d0*(10.d0**(-0.2d0*hobs(i)))/dsqrt(albedo)
       parlis0(i,1)=5.d2*dobs(i)
       volpb=volpb+(parlis0(i,1)**3)
       parlis0(i,2)=a0
       parlis0(i,3)=0.075d0
       parlis0(i,4)=2.d0*(0.5d0-ran3(idum0))
       call getom(om)
       parlis0(i,5)=om
       i=i+1
      goto 10
 20   ii=i-1
      rpb=volpb**(0.333333333333333d0)
      write(*,*)' Number of asteroids read: ',ii
      write(*,*)' Estimated size of PB  : ',((2.d-3)*rpb)
      close(1)
c -- observed size distribution
      dr=220.d0
      dd=2.d0
      nc=0
 30   continue
       dl=dr-dd
       nd=0
       do i=1,ii
        da=2.d-3*parlis0(i,1)
        if ((da.gt.dl).and.(da.lt.dr)) nd=nd+1
       enddo
       nc=nc+nd
       write(2,777)(0.5d0*(dl+dr)),nd,nc
 777   format(f10.5,2(1x,i5))
       dr=dl
      if (dr.gt.dd) goto 30
      close(2)
c - get the observed family \kapa distribution
      do i=1,ncbm
       cbin(i)=0.d0
      enddo
      da0=0.001d0
      ntrial=11
      dfkapa=4.d-6
      fkapa=0.d0
      ibinc=1
      iasteroids=0
 40   continue
       fkl=fkapa
       fkr=fkapa+dfkapa
c scan different values of a0
       do ia0=1,ntrial
        a0=3.015d0+da0*dfloat(ia0-1)
        do j=1,ii
         if (aobs(j).le.a0) then
          fk=dabs(aobs(j)-a0)/(10.d0**(0.2d0*hobs(j)))
          if ((fk.ge.fkl).and.(fk.le.fkr)) then
           iasteroids=iasteroids+1
           cbin(ibinc)=cbin(ibinc)+1.d0
          endif
         endif
        enddo
       enddo
       ibinc=ibinc+1 
       fkapa=fkapa+dfkapa
      if (fkapa.lt.1.6d-4) goto 40
c output results
      open(1,file='eoskapa.dat',status='unknown')
      fkapa=0.d0
      fntrial=dfloat(ntrial)
      ibinc=1
 50   continue
       fkl=fkapa
       fkr=fkapa+dfkapa
       fitarget(ibinc)=cbin(ibinc)/fntrial
       write(1,778)0.5*(fkl+fkr),fitarget(ibinc)
 778   format(g12.6,1x,f12.5)
       ibinc=ibinc+1 
       fkapa=fkapa+dfkapa
      if (fkapa.lt.1.6d-4) goto 50
      close(1)
      fntotast=dfloat(iasteroids)/fntrial
      write(*,*)' Effective no of asteroids: ',fntotast
      a0=3.02d0

c ........................................................
c loops over diverse parameters start...
      open(77,file='eosfit.res',status='unknown')
      dt=1.d0
      ffam=45.d0*degrad
      chi2min=1.d10
cc      a0=...

c select parameters
      timestep=1500.d0
      colfac2=1.d0

      nke=20
      do ifke=1,nke
c - choose f_KE
c      fkel=-2.3d0+(0.9d0/dfloat(nke-1))*dfloat(ifke-1)
c      fke=10.d0**fkel
      vinfmax=(100.d0/dfloat(nke-1))*dfloat(ifke-1)

      nrep=30
      do irep=1,nrep

c - initial velocity field modelled
c -- Petit and Farinella '93 model
c      rhopb=2.5d0
c      bco=0.4d0
c      bexpo=1.36d0
c      qstard=bco*rhopb*((1.d2*rpb)**bexpo)
c      vejmean2=2.d0*qstard*fke
c      vejmean=(1.d-2)*dsqrt(vejmean2)
ccc      write(*,*)' Mean ejection velocity (m/s): ',vejmean
c      vesc2=1.64d0*4.d0*pi*rhopb*1.d3*gravco*rpb*rpb/3.d0
c      vesc=dsqrt(vesc2)
ccc      write(*,*)' Escape velocity (m/s): ',vesc
ccc      write(*,*)' R_PB [km]: ',((1.d-3)*rpb)
c      vmax=750.d0
c      cffam=dcos(ffam)
c      sffam=dsin(ffam)
c      fmean=dsqrt(gms/((parlis0(1,2)*au)**3))
c      dafac=2.d0/fmean/dsqrt(1.d0-parlis(i,3)*parlis(i,3))
c      do j=1,ii
c 70    vr=vejmean*gasdev(idum0)/dsqrt(3.d0)
c       vt=vejmean*gasdev(idum0)/dsqrt(3.d0)
c       vw=vejmean*gasdev(idum0)/dsqrt(3.d0)
c       v=dsqrt(vr*vr+vt*vt+vw*vw)
c       cth=vr/v
c       cfi=vt/dsqrt(vt*vt+vw*vw)
c       sfi=vw/dsqrt(vt*vt+vw*vw)
c       if ((v.gt.vmax).or.(v.lt.vesc)) goto 70
c       vinf=dsqrt(v*v-vesc2)
c       vrinf=cth*vinf
c       vtinf=cfi*vinf*dsqrt(1.d0-cth*cth)
c       vwinf=sfi*vinf*dsqrt(1.d0-cth*cth)
c       daej=dafac*(vtinf+parlis0(i,3)*(vtinf*cffam+vrinf*sffam))/au
c       parlisini(j)=parlis0(j,2)+daej
c      enddo
c -- trivial model I
c      cffam=dcos(ffam)
c      sffam=dsin(ffam)
c      fmean=dsqrt(gms/((parlis0(1,2)*au)**3))
c      dafac=2.d0/fmean/dsqrt(1.d0-parlis(1,3)*parlis(1,3))
c      do j=1,ii
c       vinf=vinfmax*ran3(idum0)
c       cth=1.d0-2.d0*ran3(idum0)
c       phi=twopi*ran3(idum0)
c       cfi=dcos(phi)
c       sfi=dsin(phi)
c       vrinf=cth*vinf
c       vtinf=cfi*vinf*dsqrt(1.d0-cth*cth)
c       vwinf=sfi*vinf*dsqrt(1.d0-cth*cth)
c       daej=dafac*(vtinf+parlis0(j,3)*(vtinf*cffam+vrinf*sffam))/au
c       parlisini(j)=parlis0(j,2)+daej
c      enddo
c -- trivial model II
      cffam=dcos(ffam)
      sffam=dsin(ffam)
      fmean=dsqrt(gms/((parlis0(1,2)*au)**3))
      dafac=2.d0/fmean/dsqrt(1.d0-parlis(1,3)*parlis(1,3))
      jj=0
      do j=1,ii
c -- make mirror images left/right
       if (aobs(j).le.a0) then
        vrinf=vinfmax*gasdev(idum0)/dsqrt(3.d0)
        vtinf=vinfmax*gasdev(idum0)/dsqrt(3.d0)
        vwinf=vinfmax*gasdev(idum0)/dsqrt(3.d0)
        daej=dafac*(vtinf+parlis0(j,3)*(vtinf*cffam+vrinf*sffam))/au
        jj=jj+1
        parlisg(jj,2)=parlis0(j,2)+daej
        parlisini(jj)=parlisg(jj,2)
        parlisg(jj,1)=parlis0(j,1)
        parlisg(jj,3)=0.075d0
        parlisg(jj,4)=2.d0*(0.5d0-ran3(idum0))
        call getom(om)
        parlisg(jj,5)=om
        jj=jj+1
        parlisg(jj,2)=parlis0(j,2)-daej
        parlisini(jj)=parlisg(jj,2)
        parlisg(jj,1)=parlis0(j,1)
        parlisg(jj,3)=0.075d0
        parlisg(jj,4)=2.d0*(0.5d0-ran3(idum0))
        call getom(om)
        parlisg(jj,5)=om
       endif
      enddo
      fast=0.5d0*dfloat(jj)

c - choose YORP strength
      nyo=20
      do iyo=nyo,1,-1
      cyorp=(1.d0/dfloat(nyo-1))*dfloat(iyo-1)

c - copy the very-initial population to the starting state
      do i=1,jj
       parlis(i,1)=parlisg(i,1)
       parlis(i,2)=parlisg(i,2)
       parlis(i,3)=parlisg(i,3)
       parlis(i,4)=parlisg(i,4)
       parlis(i,5)=parlisg(i,5)
      enddo
      tsavini=800.d0
      dtsavini=10.d0

c evolve the system over a given timespan ...........................
c - proceed in time with all particles
      irons=0
      isty=1
      t=0.d0
 100  continue
c -- do-loop over particles (start) ..............................
       do j=1,jj
        r=parlis(j,1)
        cobl=parlis(j,4)
        om=parlis(j,5)
        taurot=colfac2*15.d0*(r**beta)
c - get thermal parameters of the objects
c        call ctypes(facks,faccs,fackd,faccd)
        surcon=5.d-3
        call stypes(surcon,facks,faccs,fackd,faccd)
c - call the mobility routines (start) .......................
c        r=664500.d0
c        om=twopi/5.d0/3600.d0
        call diur(parlis(j,2),r,om,fackd,faccd,dadtd,irons,isty)
        call seas(parlis(j,2),r,facks,faccs,dadts,irons,isty)
c    step in a...
        chaa=(dadts+cobl*(dadtd-dadts*cobl))*dt
        parlis(j,2)=parlis(j,2)+chaa
        if (parlis(j,2).lt.2.d0) then
         write(*,*)' should not happen 1'
         parlis(j,2)=2.d0
        endif
        if (parlis(j,2).gt.4.d0) then
         write(*,*)' should not happen 2'
         parlis(j,2)=4.d0
        endif
cc BLOCK OF REORIENTATIONS:
c - model YORP effect (start) ...................................
c -- initialization
c --- `nominal' YORP strength
 5      cys=gasdev(idum0)
        if (dabs(cys).gt.3.d0) goto 5
 7      cye=gasdev(idum0)
        if (dabs(cye).gt.2.d0) goto 7
        cye=cye+2.d0
        fackonv=cyorp*((1000.d0*2.5d0/r/parlis(j,2))**2)
        do k=1,91
         taus(k)=taus0(k)*fackonv*cys
         taue(k)=taue0(k)*fackonv*cye
        enddo
c -- apply
        obl=dacos(cobl)
        obld=obl*360.d0/twopi
        iobl=int(0.5d0*obld)+1
        if (iobl.lt.91) then
         de=0.5d0*(obld-eps(iobl))
         te=taue(iobl)+de*(taue(iobl+1)-taue(iobl))
         ts=taus(iobl)+de*(taus(iobl+1)-taus(iobl))
         omg=om+dt*ts
         oblg=obl+dt*te/om
         if (oblg.lt.0.d0) then
          obl=0.d0
         else
          if (oblg.gt.pi) then
           obl=pi
          else
           obl=oblg
          endif
         endif
         if (omg.lt.0.d0) then
          om=omlim
         else
          om=omg
         endif
        else
         ts=taus(91)
         omg=om+dt*ts
         if (omg.lt.0.d0) then
          om=omlim
         else
          om=omg
         endif
        endif
        peri=twopi/om/3600.d0
        parlis(j,4)=dcos(obl)
        parlis(j,5)=om
        if (om.gt.omcrit) om=omcrit
c - model YORP effect (end) .....................................
c - test reorientation of the particle (start) ..................
        taurott=taurot*((5.d0/peri/500.d0)**0.833333333)
        p=1.d0-dexp(-dt/taurott)
        xi=ran3(idum0)
        if (xi.lt.p) then
         parlis(j,4)=2.d0*(0.5d0-ran3(idum0))
         call getom(om)
         parlis(j,5)=om
        endif
c - test reorientation of the particle (end) ....................
       enddo
c -- do-loop over particles (end) ................................
c -- analyse results after some timesteps
       if (t.ge.tsavini) then
c Now compute chi2 and store the modelled \kappa-distribution
        fkapa=0.d0
        dfkapa=4.d-6
        i=1
        chi2=0.d0
        ntotasttl=0
 200    continue
         fkl=fkapa
         fkr=fkapa+dfkapa
         ntotast=0
         do j=1,jj
          if (parlis(j,2).le.a0) then
           hpart=5.d0*dlog10(1329.d3/2.d0/parlis(j,1)/dsqrt(albedo))
           fk=dabs(parlis(j,2)-a0)/(10.d0**(0.2d0*hpart))
           if ((fk.ge.fkl).and.(fk.le.fkr)) ntotast=ntotast+1
          endif
         enddo
         itargetm(i)=ntotast
c         scatarget(i)=dfloat(ntotast)*dfloat(ntotastt)/dfloat(nleft)
         ntotasttl=ntotasttl+ntotast
c - chi2 computation
         sig=dsqrt(fitarget(i))
         if (sig.lt.1.d0) sig=1.d0
         chi2=chi2+
     .        (((fitarget(i)-(dfloat(itargetm(i))*fntotast/fast))**2)
     .        /sig/sig)
c         sig=dsqrt(dfloat(itarget(i)))
c         if (sig.eq.0.d0) sig=1.d0
c         chi2=chi2+((dfloat(itarget(i)-itargetm(i))**2)/sig/sig)
c         chi2=chi2+(((dfloat(itarget(i))-scatarget(i))**2)/sig/sig)
         fkapa=fkapa+dfkapa
         i=i+1
        if (fkapa.lt.1.6d-4) goto 200
        write(77,8848)tsavini,cyorp,colfac2,vinfmax,chi2
 8848   format(f6.1,1x,f6.4,1x,f6.4,1x,f5.1,1x,f8.1)
c -- hit the best solution?
        if (chi2.lt.chi2min) then
         chi2min=chi2
         tmin=tsavini
         cymin=cyorp
         ccmin=colfac2
         vmin=vinfmax
         ntmin=ntotasttl
c --- initial population
         open(1,file='eosahbest_ini.dat',status='unknown')
         do j=1,jj
          hpart=5.d0*dlog10(1329.d3/2.d0/parlis(j,1)/dsqrt(albedo))
          write(1,779)parlisini(j),hpart
         enddo
 779     format(f6.3,1x,f7.4)
         close(1)
c --- evolved population
         open(1,file='eosahbest_fin.dat',status='unknown')
         do j=1,jj
          hpart=5.d0*dlog10(1329.d3/2.d0/parlis(j,1)/dsqrt(albedo))
          write(1,779)parlis(j,2),hpart
         enddo
         close(1)
         open(1,file='eosahbest_kapa.dat',status='unknown')
         fkapa=0.d0
         dfkapa=4.d-6
         i=1
         chi2=0.d0
 300     continue
          fkl=fkapa
          fkr=fkapa+dfkapa
          write(1,778)(0.5d0*(fkl+fkr)),(dfloat(itargetm(i))*
     .                                   fntotast/fast)
c          write(1,780)(0.5d0*(fkl+fkr)),scatarget(i)
c 780      format(g12.6,1x,f10.4)
          fkapa=fkapa+dfkapa
          i=i+1
         if (fkapa.lt.1.6d-4) goto 300
         close(1)
        endif
        tsavini=tsavini+dtsavini
       endif
       t=t+dt
      if (t.lt.timestep) goto 100

      enddo
c - closes YORP loop

      enddo
c - closes repetition loop over initial distributions

      enddo
c - closes f_KE loop

c - store the best fitting set of parameters
      write(77,*)
      write(77,*)' Best fit; chi2 = ',chi2min
      write(77,*)' T              = ',tmin
      write(77,*)' c_YORP         = ',cymin
      write(77,*)' c_coll         = ',ccmin
      write(77,*)' v_inf          = ',vmin
      write(77,*)' p_V            = ',albedo
      write(77,*)' number of asteroids = ',ntmin
      write(*,*)' Best fit; chi2 = ',chi2min
      write(*,*)' T              = ',tmin
      write(*,*)' c_YORP         = ',cymin
      write(*,*)' c_coll         = ',ccmin
      write(*,*)' v_inf          = ',vmin
      write(*,*)' p_V            = ',albedo
      write(*,*)' number of asteroids = ',ntmin
      close(77)
c stop
      stop
      end

c ==================================================================
      subroutine seas(a,r,fack,facc,dadt,irons,isty)
c *********************************************************
c This subroutine computes maximum rate of the semimajor
c axis change for the seasonal effect (ie. at 90deg
c obliquity). The input parameters are:
c
c - a   ...  semimajor axis in AU,
c - r   ...  radius in meters,
c Standard value of K=1
c
c At output you get
c dadt  ...  (da/dt) at 90deg obliquity in AU/Myr.
c
c Rem. The linear theory of the seasonal effect on spherical
c      bodies.
c .........................................................
      implicit real*8 (a-h,o-z)
c ---------------------------------------------------------
      fac=7.26525d0
      a12=dsqrt(a)
      a32=a*a12
      a34=dsqrt(a32)
c bare-basalt body
      dfac=dsqrt(fack)
      dfacc=dsqrt(facc)
      thetag=0.19811d0*a34*dfac*dfacc
      scal=1.4527d0*a34*dfac/dfacc
      if (irons.eq.0) then
       if (isty.eq.1) then
        rho3=2.5d0
       else
        rho3=1.5d0
       endif
      else
       rho3=7.5d0      
      endif
      rls=r/scal
      al=dsqrt(2.d0)*rls
      cal=dcos(al)
      sal=dsin(al)
      if (al.lt.90.d0) then
       ealm=dexp(-al)
      else
       ealm=0.d0
      endif
      tau=thetag/al
      tau1=1.d0+tau
      te1=3.d0*(al+2.d0)*ealm+(3.d0*(al-2.d0)*cal+al*(al-3.d0)*sal)
      te2=al*(al+3.d0)*ealm+(-al*(al-3.d0)*cal+3.d0*(al-2.d0)*sal)
      te3=-(al+2.d0)*ealm+(-(al-2.d0)*cal+al*sal)
      te4=-al*ealm-(al*cal+(al-2.d0)*sal)
      capa=te3
      capb=te4
      capc=te3+tau*te1/tau1
      capd=te4+tau*te2/tau1
c E sin(d) & E cos(d) ...
      dtt1=-0.222222222222d0*(capb*capc-capa*capd)/(capc*capc+
     . capd*capd)/tau1
      dadt=-7.26525d0*dtt1/rho3/r/a12
      return
      end
c ==================================================================
      subroutine diur(a,r,omega,fack,facc,dadt,irons,isty)
c *********************************************************
c This subroutine computes maximum rate of the semimajor
c axis change for the diurnal effect (ie. at 0deg
c obliquity). The input parameters are:
c
c - a   ...  semimajor axis in AU,
c - r   ...  radius in meters,
c   Standard value K=0.0015
c
c At output you get
c dadt  ...  (da/dt) at 0deg obliquity in AU/Myr.
c
c Rem. The linear theory of the seasonal effect on spherical
c      bodies. Rotation frequency is supposed to be
c      inversely proportional to the radius of the fragment
c      (as in Farinella etal, 1998).
c .........................................................
      implicit real*8 (a-h,o-z)
c ---------------------------------------------------------
c Constants
      faca=7.26525d0
      a12=dsqrt(a)
      a32=a*a12
c Theta and sqrt(2)*R':
      dfac=dsqrt(fack)
      dfacc=dsqrt(facc)
      sqrto=dsqrt(omega)
      thpar=11.2705183d0*a32*dfac*dfacc*sqrto
      al=36878.1778d0*r*dfacc*sqrto/dfac
      if (irons.eq.0) then
       if (isty.eq.1) then
        rho3=2.5d0
       else
        rho3=1.5d0
       endif
      else
       rho3=7.5d0      
      endif
      tau=thpar/al
      tau1=1.d0+tau
      cal=dcos(al)
      sal=dsin(al)
      if (al.lt.90.d0) then
       ealm=dexp(-al)
      else
       ealm=0.d0
      endif
      te1=3.d0*(al+2.d0)*ealm+(3.d0*(al-2.d0)*cal+al*(al-3.d0)*sal)
      te2=al*(al+3.d0)*ealm+(-al*(al-3.d0)*cal+3.d0*(al-2.d0)*sal)
      te3=-(al+2.d0)*ealm+(-(al-2.d0)*cal+al*sal)
      te4=-al*ealm-(al*cal+(al-2.d0)*sal)
      capa=te3
      capb=te4
      capc=te3+tau*te1/tau1
      capd=te4+tau*te2/tau1
c E sin(d) & E cos(d)...
      psi1=-0.444444444444d0*(capb*capc-capa*capd)/(capc*capc+
     . capd*capd)/tau1
      dadt=7.26525d0*psi1/r/rho3/a12
      return
      end
c ===============================================================
      subroutine stypes(surcon,facks,faccs,fackd,faccd)
c thermal parameters of S-types
      implicit double precision(a-h,o-z)
        parameter (tiny=1.d-5)
c ---------------------------------------------------------------
c Thermal constant to be specified here
c (reference values: K_ref=1, C_ref=680, rho_ref=3.5 for
c                    the seasonal variant and
c                    K_ref=0.0015, C_ref=680, rho_ref=1.5 for
c                    the diurnal variant)
c stony object with the surface conductivity K=0.002-2 W/m/K
      if ((surcon.lt.0.0015d0-tiny).or.(surcon.gt.2.d0+tiny)) then
       write(*,*)' IN YARKO SUBROUTINE:'
       write(*,*)' Surface conductivity of the stony bodies'
       write(*,*)' out of the 0.0015-2 range;'
       write(*,*)' surcon = ',surcon
       stop
      endif
c - an approximate \rho(K) dependence
c      rhomin=1.5d0
c      rhomax=3.d0
c      surmin=0.0015d0
c      surmax=2.d0
c      rho=rhomin+(rhomax-rhomin)*dlog10(surcon/surmin)/
c     .    dlog10(surmax/surmin)
      rho=2.d0
c - seasonal effect parameters
      surconref=1.d0
      rhoref=3.5d0
      facks=surcon/surconref
      faccs=rho/rhoref
c - diurnal effect parameters
      surconref=0.0015d0
      rhoref=1.9d0
      fackd=surcon/surconref
      faccd=rho/rhoref
      return
      end
c ===============================================================
      subroutine ctypes(facks,faccs,fackd,faccd)
c thermal parameters of C-types
      implicit real*8 (a-h,o-z)
c ---------------------------------------------------------------
c Thermal constant to be specified here
c (reference values: K_ref=1, C_ref=680, rho_ref=3.5 for
c                    the seasonal variant and
c                    K_ref=0.0015, C_ref=680, rho_ref=1.5 for
c                    the diurnal variant)
c guessed thermal parameters of the C-type objects
c K = 0.01 , C = 1500, rho = 1.5
      facks=0.01d0
      faccs=1.5d0*1500.d0/3.5d0/680.d0
      fackd=0.01d0/0.0015d0
      faccd=1.5d0*1500.d0/1.5d0/680.d0
      return
      end
c ===============================================================
      subroutine getom(om)
      implicit real*8 (a-h,o-z)
      common /co/ twopi,ompr,ds3,rotmin,rotmax      
 5    continue
       om1=ompr*gasdev(idum0)/ds3
       om2=ompr*gasdev(idum0)/ds3
       om3=ompr*gasdev(idum0)/ds3
       om=dsqrt(om1*om1+om2*om2+om3*om3)
       rot=twopi/om
      if ((rot.lt.rotmin).or.(rot.gt.rotmax)) goto 5
      return
      end
c ===============================================================
      function ran3(idum)
      implicit real*8 (a-h,o-z)
      parameter (mbig=1000000000,mseed=161803398,mz=0,fac=1.d-9)
      dimension ma(55)
      data iff /0/
      save
c --------------------------------------------------------------------
      if((idum.ge.0).and.(iff.ne.0)) goto 20
        iff=1
        mj=mseed-iabs(idum)
        mj=mod(mj,mbig)
        ma(55)=mj
        mk=1
        do 11 i=1,54
          ii=mod(21*i,55)
          ma(ii)=mk
          mk=mj-mk
          if(mk.lt.mz)mk=mk+mbig
          mj=ma(ii)
11      continue
        do 13 k=1,4
          do 12 i=1,55
            ma(i)=ma(i)-ma(1+mod(i+30,55))
            if(ma(i).lt.mz)ma(i)=ma(i)+mbig
12        continue
13      continue
        inext=0
        inextp=31
        idum=1
20    continue
      inext=inext+1
      if(inext.eq.56)inext=1
      inextp=inextp+1
      if(inextp.eq.56)inextp=1
      mj=ma(inext)-ma(inextp)
      if(mj.lt.mz)mj=mj+mbig
      ma(inext)=mj
      ran3=mj*fac
      return
      end
c ============================================================
      function gasdev(idumm)
      implicit real*8 (a-h,o-z)
      data iset /0/
      save
c ------------------------------------------------------------
      if (iset.eq.0) then
1      v1=2.d0*ran3(idumm)-1.e0
       v2=2.d0*ran3(idumm)-1.e0
       r=v1*v1+v2*v2
       if (r.ge.1.d0) goto 1
       fac=dsqrt(-2.d0*dlog(r)/r)
       gset=v1*fac
       gasdev=v2*fac
       iset=1
      else
       gasdev=gset
       iset=0
      endif
      return
      end
